#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <iostream>
#include "song.h"
#include "playsong.h"
#include <stdlib.h>

int seqfd = -1;
SEQ_DEFINEBUF(1024);

void seqbuf_dump();




int midiDevice;
int rate;

int main(int argc, char *argv[])
	{

  std::cout << std::endl;
	std::cout << "---===      Ryth(M)aid v0.0001     ===---" << std::endl;
	std::cout << "---===      by Florian Schmidt     ===---" << std::endl;
	std::cout << "---===       a.k.a MistaTapas      ===---" << std::endl;
	std::cout << "---=== (c) in 2001 Florian Schmidt ===---" << std::endl<<std::endl;
	std::cout << "usage: 'rythmaid [Device] (e.g. /dev/sequencer) [midiPort] [Songfile]'" << std::endl;
	std::cout << "use 'rythmaid [Device]' to list MidiPorts" << std::endl<<std::endl;

	if (argc < 2)
		{
		exit(0);
		}

	if (  (seqfd = open (argv[1], O_WRONLY,0)) == -1 )
		{
		perror ("oh boy, this stinks. couldn't open device");
		exit (-1);
		}

	std::cout << "succesfully opened" << argv[1] << std::endl;

	// get number of Devices, but only midi is of interes to us..
	int numberOfSynths;
	int numberOfMidis;
	ioctl (seqfd, SNDCTL_SEQ_NRSYNTHS, &numberOfSynths);
	ioctl (seqfd, SNDCTL_SEQ_NRMIDIS, &numberOfMidis);
	std::cout << "number of Synths: " << numberOfSynths << std::endl;
	std::cout << "number of MidiPorts: " << numberOfMidis << std::endl;
		
	// check the names of the midi ports...
	midi_info myMidiInfo;
	for (int i = 0; i < numberOfMidis; i++)
		{
		myMidiInfo.device = i;	
		ioctl (seqfd, SNDCTL_MIDI_INFO, &myMidiInfo);
		std::cout << i << ": " << myMidiInfo.name << " type: " << myMidiInfo.dev_type << std::endl;
		}

		
	// determine the timer resolution
	ioctl (seqfd, SNDCTL_SEQ_CTRLRATE, &rate);
	std::cout << "rate : 1/" << rate << std::endl;


	// this stuff we execute, if the user has specified the necessary options
	if (argc == 4)	
		{
		midiDevice = atoi(argv[2]);

		if (midiDevice > (numberOfMidis - 1))
			{
			cout << "Error: midiPort not available" << std:: endl;
			if ( (close (seqfd)) == -1)
				{
				perror ("ouch. couldn't close device");
				}
			exit (-1);
			}

		std::cout << "playing Song" << std::endl;	
		std::cout << "using device   : " << argv[1] <<std::endl;
		std::cout << "using midiPort : " << midiDevice <<std::endl;
		std::cout << "using Songfile : " << argv[3] <<std::endl;
		Song mySong;

		int myError;
		if ( (myError = mySong.LoadSong(argv[3])) != 0)
			{
			std::cout << "Error! Loading of song failed!" << std::endl;
			std::cout << "error: " << myError << std::endl;
			if ( (close (seqfd)) == -1)
				{
				perror ("ouch. couldn't close device");
				}
			exit (-1);
			}
			
		playSong(&mySong);
		}

	if ( (close (seqfd)) == -1)
		{
		perror ("ouch. couldn't close device");
		exit (-1);
		}

	std::cout << "succesfully closed /dev/sequencer" << std::endl;

	std::cout << std::endl;
	}
	

void seqbuf_dump()
	{
	if (_seqbufptr)
		{
		if (write (seqfd, _seqbuf, _seqbufptr) == 1)
			{
			perror ("write on /dev/sequencer failed");
			exit(-1);
			}
		}
	_seqbufptr = 0;
	}	
